# coding:utf-8
from PySide6.QtCore import QEasingCurve, QPropertyAnimation, Qt, QEvent
from PySide6.QtGui import QColor, QResizeEvent
from PySide6.QtWidgets import (QDialog, QGraphicsDropShadowEffect,
                             QGraphicsOpacityEffect, QHBoxLayout, QWidget, QFrame)

from ...common.config import isDarkTheme
from ...common.enum import QtEnum


class MaskDialogBase(QDialog):
    """ Dialog box base class with a mask """

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self._hBoxLayout = QHBoxLayout(self)
        self.windowMask = QWidget(self)

        # dialog box in the center of mask, all widgets take it as parent
        self.widget = QFrame(self, objectName='centerWidget')
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setGeometry(0, 0, parent.width(), parent.height())

        c = 0 if isDarkTheme() else 255
        self.windowMask.resize(self.size())
        self.windowMask.setStyleSheet(f'background:rgba({c}, {c}, {c}, 0.6)')
        self._hBoxLayout.addWidget(self.widget)
        self.setShadowEffect()

        self.window().installEventFilter(self)

    def setShadowEffect(self, blurRadius=60, offset=(0, 10), color=QColor(0, 0, 0, 100)):
        """ add shadow to dialog """
        shadowEffect = QGraphicsDropShadowEffect(self.widget)
        shadowEffect.setBlurRadius(blurRadius)
        shadowEffect.setOffset(*offset)
        shadowEffect.setColor(color)
        self.widget.setGraphicsEffect(None)
        self.widget.setGraphicsEffect(shadowEffect)

    def setMaskColor(self, color: QColor):
        """ set the color of mask """
        self.windowMask.setStyleSheet(f"""
            background: rgba({color.red()}, {color.blue()}, {color.green()}, {color.alpha()})
        """)

    def showEvent(self, e):
        """ fade in """
        opacityEffect = QGraphicsOpacityEffect(self)
        self.setGraphicsEffect(opacityEffect)
        opacityAni = QPropertyAnimation(opacityEffect, b'opacity', self)
        opacityAni.setStartValue(0)
        opacityAni.setEndValue(1)
        opacityAni.setDuration(200)
        opacityAni.setEasingCurve(QEasingCurve.InSine)
        opacityAni.finished.connect(opacityEffect.deleteLater)
        opacityAni.start()
        super().showEvent(e)

    def done(self, code):
        """ fade out """
        self.widget.setGraphicsEffect(None)
        opacityEffect = QGraphicsOpacityEffect(self)
        self.setGraphicsEffect(opacityEffect)
        opacityAni = QPropertyAnimation(opacityEffect, b'opacity', self)
        opacityAni.setStartValue(1)
        opacityAni.setEndValue(0)
        opacityAni.setDuration(100)
        opacityAni.finished.connect(lambda: QDialog.done(self, code))
        opacityAni.start()

    def resizeEvent(self, e):
        self.windowMask.resize(self.size())

    def eventFilter(self, obj, e: QEvent):
        if obj is self.window():
            if e.type() == QtEnum.QEvent.Resize:
                re = QResizeEvent(e)
                self.resize(re.size())

        return super().eventFilter(obj, e)



from PySide6.QtCore import Qt, QEvent, QPropertyAnimation, QFile, QEasingCurve
from PySide6.QtGui import QPainter, QPalette, QIcon, QKeySequence
from PySide6.QtWidgets import QStyle


# Fix memory leak
class QtEnum:
    class Qt:
        NoPen = Qt.NoPen
        white = Qt.white
        black = Qt.black
        transparent = Qt.transparent
        AlignCenter = Qt.AlignCenter
        ForegroundRole = Qt.ForegroundRole
        CheckStateRole = Qt.CheckStateRole
        DisplayRole = Qt.DisplayRole
        FontRole = Qt.FontRole
        DecorationRole = Qt.DecorationRole
        UserRole = Qt.UserRole
        AlignLeft = Qt.AlignLeft
        AlignRight = Qt.AlignRight
        AlignVCenter = Qt.AlignVCenter
        LeftButton = Qt.LeftButton
        NoItemFlags = Qt.NoItemFlags
        Horizontal = Qt.Horizontal
        Popup = Qt.Popup
        FramelessWindowHint = Qt.FramelessWindowHint
        NoDropShadowWindowHint = Qt.NoDropShadowWindowHint
        WA_TranslucentBackground = Qt.WA_TranslucentBackground
        Vertical = Qt.Vertical
        Orientation = Qt.Orientation
        WA_StyledBackground = Qt.WA_StyledBackground
        WA_TransparentForMouseEvents = WA_TransparentForMouseEvents
    class QEasingCurve:
        OutQuad = QEasingCurve.OutQuad
    class QStyle:
        State_Enabled = QStyle.State_Enabled
        State_MouseOver = QStyle.State_MouseOver
    class QKeySequence:
        NativeText = QKeySequence.NativeText
    class QIcon:
        Disabled = QIcon.Disabled
        Selected = QIcon.Selected
        Normal = QIcon.Normal

    class QFile:
        ReadOnly = QFile.ReadOnly

    class QPalette:
        Text = QPalette.Text
        HighlightedText = QPalette.HighlightedText

    class QPropertyAnimation:
        Running = QPropertyAnimation.Running
        class State:
            Running = QPropertyAnimation.State.Running
            Stopped = QPropertyAnimation.State.Stopped

    class QPainter:
        Antialiasing = QPainter.Antialiasing
        TextAntialiasing = QPainter.TextAntialiasing
        SmoothPixmapTransform = QPainter.SmoothPixmapTransform

    class QFile:
        ReadOnly = QFile.ReadOnly

    class QEvent:
        WindowStateChange = QEvent.WindowStateChange
        Resize = QEvent.Resize
        MouseButtonPress = QEvent.MouseButtonPress
        MouseButtonRelease = QEvent.MouseButtonRelease
        Enter = QEvent.Enter
        Leave = QEvent.Leave
        Wheel = QEvent.Wheel
        Show = QEvent.Show

        class Type:
            EnabledChange = QEvent.Type.EnabledChange
            Enter = QEvent.Type.Enter
            Leave = QEvent.Type.Leave
            MouseButtonPress = QEvent.Type.MouseButtonPress
